import typing

APPS = ["registry", "datastore", "vis"]
APPS_TYPE = typing.Literal["registry", "datastore", "vis"]
API_DEV_URL = "http://0.0.0.0:8042/api/"
API_PROD_URL = "https://api.mosqlimate.org/api/"
